package org.simplericity.jettyconsole.jsp;

import org.apache.jasper.servlet.JspServlet;
import org.eclipse.jetty.webapp.AbstractConfiguration;
import org.eclipse.jetty.webapp.WebAppContext;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Map;

/**
 * Glassfish JSP / Jasper is too jar files based in a few cases.
 * This Jetty configuration postConfigures the Jasper JSP servlet
 * to not use redeployMode for Jasper's TldLocationsCache.
 *
 * We also preemptively run TldLocationsCache's init method, collect the
 * <code>mappings</code> map and replace all <code>location</code> values
 * such that TLD's under /META-INF/jettyconsole/lib/* can be found by Jasper's.
 * TagLibraryInfoImpl.
 *
 * All of this done using reflection. Ugly bugly.
 *
 */
public class JasperInUnpackedWarUglyHackConfiguration extends AbstractConfiguration {

    @Override
    public void postConfigure(WebAppContext webAppContext) throws Exception {

        // Grab the JSP servlet
        JspServlet servlet = (JspServlet) webAppContext.getServletHandler().getServlet("jsp").getServletInstance();

        // Get the EmbeddedServletOptions options object
        Field optionsField = JspServlet.class.getDeclaredField("options");
        optionsField.setAccessible(true);
        Object optionsObject = optionsField.get(servlet);

        // Get the TldLocationsCache tldLocationsCache object
        Field cacheField = optionsObject.getClass().getDeclaredField("tldLocationsCache");
        cacheField.setAccessible(true);
        Object cacheObject = cacheField.get(optionsObject);


        // Set TldLocationsCache.redeployMode to false
        Field redeployModeField = cacheObject.getClass().getDeclaredField("redeployMode");
        redeployModeField.setAccessible(true);
        redeployModeField.set(cacheObject, Boolean.FALSE);

        // Run TldLocationsCache.init()
        ClassLoader contextLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(webAppContext.getClassLoader());
        try {
            Method initMethod = cacheObject.getClass().getDeclaredMethod("init");
            initMethod.setAccessible(true);
            initMethod.invoke(cacheObject);

        } finally {
            if(contextLoader != null) {
                Thread.currentThread().setContextClassLoader(contextLoader);
            }
        }
        // Replace {"file://path/file.war", "META-INF/jettyconsole/*.tld"} with {"/META-INF/jettyconsole/*.tld"}
        Field mappingsField = cacheObject.getClass().getDeclaredField("mappings");
        mappingsField.setAccessible(true);

        Map<String, String[]> map = (Map<String, String[]>) mappingsField.get(cacheObject);

        for(String name : new HashSet<String>(map.keySet())){
            String[] location = map.get(name);
            if(location.length == 2 && location[1].startsWith("META-INF/jettyconsole/lib")) {
                map.put(name, new String[] {"/" +location[1], null});
            }
        }

    }
}
